float4x4 gWorldViewProj : WorldViewProjection;
float4x4 gViewInverse : ViewInverse;
Texture2D gParticleTexture;

SamplerState samPoint
{
	Filter = MIN_MAG_MIP_POINT;
	AddressU = WRAP;
	AddressV = WRAP;
};

//STATES
//******
BlendState AlphaBlending
{
	BlendEnable[0] = TRUE;
	SrcBlend = SRC_ALPHA;
	DestBlend = INV_SRC_ALPHA;
	BlendOp = ADD;
	SrcBlendAlpha = ONE;
	DestBlendAlpha = ZERO;
	BlendOpAlpha = ADD;
	RenderTargetWriteMask[0] = 0x0f;
};

DepthStencilState DisableDepthWriting
{
	//Enable Depth Rendering
	//Disable Depth Writing
	DepthWriteMask = ZERO;
	DepthEnable = true;

};

RasterizerState BackCulling
{
	CullMode = BACK;
};


//SHADER STRUCTS
//**************
struct VS_DATA
{
	float3 Position : POSITION;
	float4 Color: COLOR;
	float Size : TEXCOORD0;
	float Rotation : TEXCOORD1;
};

struct GS_DATA
{
	float4 Position : SV_POSITION;
	float2 TexCoord: TEXCOORD0;
	float4 Color : COLOR;
};

//VERTEX SHADER
//*************
VS_DATA MainVS(VS_DATA input)
{
	return input;
}

//GEOMETRY SHADER
//***************
void CreateVertex(inout TriangleStream<GS_DATA> triStream, float3 pos, float2 texCoord, float4 col)
{
	//Step 1. Create a GS_DATA object
	//Step 2. Transform the position using the WVP Matrix and assign it to (GS_DATA object).Position (Keep in mind: float3 -> float4, Homogeneous Coordinates)
	//Step 3. Assign texCoord to (GS_DATA object).TexCoord
		//This is a little formula to do texture rotation by transforming the texture coordinates (Can cause artifacts)
		/*texCoord -= float2(0.5f,0.5f);
		texCoord = mul(texCoord, uvRotation);
		texCoord += float2(0.5f,0.5f);*/
		//Step 4. Assign color to (GS_DATA object).Color
		//Step 5. Append (GS_DATA object) to the TriangleStream parameter (TriangleStream::Append(...))

	GS_DATA output = (GS_DATA)0;

	output.Position = mul(float4(pos, 1.f), gWorldViewProj);

	output.TexCoord = texCoord;
	output.Color = col;

	triStream.Append(output);

}

[maxvertexcount(4)]
void MainGS(point VS_DATA vertex[1], inout TriangleStream<GS_DATA> triStream)
{
	//Use these variable names
	float3 topLeft, topRight, bottomLeft, bottomRight;
	float size = vertex[0].Size;
	float3 origin = vertex[0].Position;

	//rotate the quad
	float halfSize = size / 2;

	float2 tl = float2(- halfSize, halfSize);
	float2 tr = float2(halfSize, halfSize);
	float2 bl = float2(- halfSize, - halfSize);
	float2 br = float2(halfSize, - halfSize);

	float2x2 rot = { cos(vertex[0].Rotation), -sin(vertex[0].Rotation), sin(vertex[0].Rotation), cos(vertex[0].Rotation) };

	tl = mul(tl, rot);
	tr = mul(tr, rot);
	bl = mul(bl, rot);
	br = mul(br, rot);

	topLeft = float3(tl, 1.f);
	topRight = float3(tr, 1.f);
	bottomLeft = float3(bl , 1.f);
	bottomRight = float3(br, 1.f);


	//Transform the vertices using the ViewInverse (Rotational Part Only!!! (~ normal transformation)), this will force them to always point towards the camera (cfr. BillBoarding)
	topLeft = mul(topLeft, (float3x3)gViewInverse);
	topRight = mul(topRight, (float3x3)gViewInverse);
	bottomLeft = mul(bottomLeft, (float3x3)gViewInverse);
	bottomRight = mul(bottomRight, (float3x3)gViewInverse);

	topLeft = topLeft + origin;
	topRight = topRight + origin;
	bottomLeft = bottomLeft + origin;
	bottomRight = bottomRight + origin;



	//Create Geometry (Trianglestrip)
	CreateVertex(triStream, bottomLeft, float2(0, 1), vertex[0].Color);
	CreateVertex(triStream, topLeft, float2(0, 0), vertex[0].Color);
	CreateVertex(triStream, bottomRight, float2(1, 1), vertex[0].Color);
	CreateVertex(triStream, topRight, float2(1, 0), vertex[0].Color);
}

//PIXEL SHADER
//************
float4 MainPS(GS_DATA input) : SV_TARGET{

	//Simple Texture Sampling
	float4 result = gParticleTexture.Sample(samPoint,input.TexCoord);
	return input.Color * result;
}

// Default Technique
technique10 Default {

	pass p0 {
		SetVertexShader(CompileShader(vs_4_0, MainVS()));
		SetGeometryShader(CompileShader(gs_4_0, MainGS()));
		SetPixelShader(CompileShader(ps_4_0, MainPS()));

		SetRasterizerState(BackCulling);
		SetDepthStencilState(DisableDepthWriting, 0);
		SetBlendState(AlphaBlending, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);
	}
}
